import "dart:html";

class DaySection
{  
  DaySection(this._color, this._timeStart, this._timeEnd, [this._extra = null]);
  
  DivElement spawn(DateTime date)
  {
    _date = date.toIso8601String();
    _date = _date.substring(0, _date.indexOf("T"));
    DateTime timeStartDate = DateTime.parse(_date + " " + _timeStart);            
    DateTime timeEndDate = DateTime.parse(_date + " " + _timeEnd);
     
    int minutes = timeEndDate.difference(timeStartDate).inMinutes;
    int sectionTop = (0.5 * (timeStartDate.difference(DateTime.parse(_date + " 00:00")).inMinutes - 360)).round();
    int sectionHeight = (0.5 * minutes).round();
     
    _element = new DivElement();
    _element.style.position = "absolute";
    _element.style.top = sectionTop.toString() + "px";
    _element.style.width = "100%";
    _element.style.display = "block";
    _element.style.backgroundColor = _color;
    _element.style.opacity = "0.3";
    _element.style.height = sectionHeight.toString() + "px";
    _element.style.borderTop = _element.style.borderBottom = "1px solid black";
    return _element;
  }
    
  String get date => _date;
  DivElement get element => _element;
  Map<String, String> get extra => _extra;
  String get timeEnd => _timeEnd;
  String get timeStart => _timeStart;
  
  String _color;
  String _timeStart;
  String _timeEnd;
  String _date;
  Map<String, String> _extra;
  DivElement _element;
}

class Day
{
  Day(this._date)
  {
    _content = new DivElement();
  }
  
  void clear()
  {
    _content.children.clear();    
  }
  
  /// Add a new DaySection (specifying a set of hours:minutes) to the Day, ignoring DaySections with start/end times similar to any DaySections already present
  /// in the day. DaySections are automatically sorted chronologically based on their StartTimes. 
  void addSection(DaySection section)
  {
    if (section == null) throw new ArgumentError.notNull("section");    
    
    DivElement sectionDiv = section.spawn(_date);
    DateTime newSectionStartTime = DateTime.parse(section.date + " " + section.timeStart);
    DateTime newSectionEndTime = DateTime.parse(section.date + " " + section.timeEnd);
            
    /// Don't add the section if another section with the same start/end time is already present in the Day 
    DaySection compareSection = _sections.firstWhere((DaySection s)
    {
      DateTime compareSectionStartTime = DateTime.parse(s.date + " " + s.timeStart);
      DateTime compareSectionEndTime = DateTime.parse(s.date + " " + s.timeEnd);
      return (compareSectionStartTime.compareTo(newSectionStartTime) == 0 && compareSectionEndTime.compareTo(newSectionEndTime) == 0);
    }, orElse: () => null);
    if (compareSection != null) return;
    
    /// We want the sections to be added in chronological order - find index of first section with higher starttime than the new section
    compareSection = _sections.firstWhere((DaySection s)
    {
      DateTime dt = DateTime.parse(s._date + " " + s.timeStart);
      return (dt.compareTo(newSectionStartTime) > 0);
    }, orElse: () => null);
    
    if (compareSection == null)
    {
      _sections.add(section);
      _content.children.add(sectionDiv);
    }
    
    else
    {
      int index = _sections.indexOf(compareSection);
      _sections.insert(index, section);
      _content.children.insert(index, sectionDiv);
    }   
  }
 
  List<DaySection> get sections => _sections;
  
  final DateTime _date;
  DivElement _content;
  DivElement get content => _content;
  List<DaySection> _sections = new List();
}
